<?php
class Wp_Image_compression {

    private $id;
    private $compression_settings      = array();
    private $thumbs_data               = array();
    private $backup_before_compression = 0;

    function __construct() {

        if( ! class_exists('Optimisationio_Dashboard') ){
			require_once 'class-optimisationio-dashboard.php';
		}

		Optimisationio_Dashboard::init();

        require_once 'class-wp-image.php';

        $this->compression_settings = unserialize( get_option('_wpimage_options') );

        $this->backup_before_compression = isset( $this->compression_settings['backup_before_compression'] ) ? $this->compression_settings['backup_before_compression'] : false;

        add_action('admin_enqueue_scripts', array(&$this, 'admin_scripts'));
        add_action('wp_ajax_wpimage_request', array(&$this, 'wpimage_media_library_ajax_callback'));
        add_action('manage_media_custom_column', array(&$this, 'fill_media_columns'), 10, 2);
        add_filter('manage_media_columns', array(&$this, 'add_media_columns'));
        //add_filter('wp_generate_attachment_metadata', array(&$this, 'optimize_thumbnails'));
        add_action('add_attachment', array(&$this, 'wpimage_media_uploader_callback'));

        add_action( 'wp_loaded', array( $this, 'exe_before_wpheader' ) );

        if ( ! is_admin() ) {
            //lazyl
            global $wpImagelazySizesDefaults;

            $settings = get_option( '_wpimage_lazyload_options', $wpImagelazySizesDefaults);

            if (isset($settings) && $settings != '') {
               $settings = unserialize($settings);
            }

            if (isset($settings['enable_lazyload']) && $settings['enable_lazyload'] == 'true') {
               add_action( 'wp_enqueue_scripts', array( &$this, 'add_lazyloading_styles' ), 1 );
               add_action( 'wp_enqueue_scripts', array( &$this, 'add_lazyloading_scripts' ), 200 );
               // Run this later, so other content filters have run, including image_add_wh on WP.com
               add_filter( 'the_content', array( &$this, 'filter_images'), 200 );
               add_filter( 'post_thumbnail_html', array( &$this, 'filter_images'), 200 );
               add_filter( 'widget_text', array( &$this, 'filter_images'), 200 );
               if ($settings['lazyload_iframe'] != 'false') {
                   add_filter('oembed_result', array(&$this, 'filter_iframes'), 200);
                   add_filter('embed_oembed_html', array(&$this, 'filter_iframes'), 200);
               }
               add_filter('get_avatar', array(&$this, 'filter_avatar'), 200);
            }
        }
    }

    private function get_js_config() {
        global $wpImagelazySizesDefaults;
        $settings = unserialize( get_option( '_wpimage_lazyload_options', $wpImagelazySizesDefaults ));
        return array(
            'expand' => $settings['lazyload_expand'],
            'preloadAfterLoad' => $settings['lazyload_preloadAfterLoad']
            );
    }

    private function _add_class( $htmlString = '', $newClass ) {

        $pattern = '/class="([^"]*)"/';

        // Class attribute set.
        if ( preg_match( $pattern, $htmlString, $matches ) ) {
            $definedClasses = explode( ' ', $matches[1] );
            if ( ! in_array( $newClass, $definedClasses, true ) ) {
                $definedClasses[] = $newClass;
                $htmlString = str_replace(
                    $matches[0],
                    sprintf( 'class="%s"', implode( ' ', $definedClasses ) ),
                    $htmlString
                );
            }
        // Class attribute not set.
        } else {
            $htmlString = preg_replace( '/(\<.+\s)/', sprintf( '$1class="%s" ', $newClass ), $htmlString );
        }

        return $htmlString;
    }

    private function validate_cloudinary_credentials( $cloud_name = '', $api_key = '', $api_secret = '' ){
        include_once 'lib/Api.php';
        include_once 'lib/Cloudinary.php';
        $ret = false;
        if( '' === $cloud_name || '' === $api_key || '' === $api_secret ){
            return $ret;
        }
        try{
            \Cloudinary::config(
                array(
                    "cloud_name"=> $cloud_name,
                    "api_key" => $api_key,
                    "api_secret" => $api_secret
                )
            );
            $api = new \Cloudinary\Api();
            $ping = $api->ping();
            $ret = $ping && isset( $ping['status'] ) && 'ok' === $ping['status'];
        }
        catch (Exception $e) {}
        return $ret;
    }

    /*
     *  Adds wpimage fields and settings to Settings->Media settings page
     */
    public function add_lazyloading_styles() {
        wp_enqueue_style( 'wpimage_lazyloading', plugins_url( ltrim( 'css/lazysizes.min.css', '/' ), __FILE__ ), array(), "1.1" );
    }

    public function add_lazyloading_scripts() {
        global $wpImagelazySizesDefaults;
        $settings = unserialize(get_option( '_wpimage_lazyload_options', $wpImagelazySizesDefaults));
        wp_enqueue_script( 'wpimage_lazyloading', plugins_url( ltrim( 'build/wp-lazysizes.min.js', '/' ), __FILE__ ), array(), "1.1", false );

        if ( $settings['wpimage_optimumx'] !== 'false' ) {
            wp_enqueue_script( 'lazysizesoptimumx',
               plugins_url( ltrim( 'js/lazysizes/plugins/optimumx/ls.optimumx.min.js', '/' ), __FILE__ ), array(), "1.1", false );
        }

        wp_localize_script( 'wpimage_lazyloading', 'lazySizesConfig', $this->get_js_config() );
    }

    public function filter_images( $content, $type = 'ratio' ) {
        global $wpImagelazySizesDefaults;
        $settings = unserialize(get_option( '_wpimage_lazyload_options', $wpImagelazySizesDefaults));
        if ( is_feed()
            || intval( get_query_var( 'print' ) ) == 1
            || intval( get_query_var( 'printpage' ) ) == 1
            || strpos( $_SERVER['HTTP_USER_AGENT'], 'Opera Mini' ) !== false
        ) {
            return $content;
        }

        $respReplace = 'data-sizes="auto" data-srcset=';

        if ( $settings['lazyload_optimumx'] != 'false' ) {
            $respReplace = 'data-optimumx="' . $settings['lazyload_optimumx'] . '" ' . $respReplace;
        }

        $matches = array();
        $skip_images_regex = '/class=".*lazyload.*"/';
        $placeholder_image = apply_filters( 'lazysizes_placeholder_image',
            'data:image/gif;base64,R0lGODlhAQABAAAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==' );
        preg_match_all( '/<img\s+.*?>/', $content, $matches );

        $search = array();
        $replace = array();

        foreach ( $matches[0] as $imgHTML ) {

            // Don't to the replacement if a skip class is provided and the image has the class.
            if ( ! ( preg_match( $skip_images_regex, $imgHTML ) ) ) {

                $replaceHTML = preg_replace( '/<img(.*?)src=/i',
                    '<img$1src="' . $placeholder_image . '" data-src=', $imgHTML );

                $replaceHTML = preg_replace( '/srcset=/i', $respReplace, $replaceHTML );

                $replaceHTML = $this->_add_class( $replaceHTML, 'lazyload' );

                $replaceHTML .= '<noscript>' . $imgHTML . '</noscript>';

                if ( $type == 'ratio' && $settings['lazyload_intrinsicRatio'] != 'false' ) {
                    if ( preg_match( '/width=["|\']*(\d+)["|\']*/', $imgHTML, $width ) == 1
                        && preg_match( '/height=["|\']*(\d+)["|\']*/', $imgHTML, $height ) == 1 ) {

                        $ratioBox = '<span class="intrinsic-ratio-box';
                        if ( preg_match( '/(align(none|left|right|center))/', $imgHTML, $align_class ) == 1 ) {
                            $ratioBox .= ' ' . $align_class[0];
                            $replaceHTML = str_replace( $align_class[0], '', $replaceHTML );
                        }
                        if ( $settings['lazyload_intrinsicRatio'] == 'animated' ) {
                            $ratioBox .= ' lazyload" data-expand="-1';
                        }

                        $ratioBox .= '" style="max-width: ' . $width[1] . 'px; max-height: ' . $height[1] . 'px;';

                        $ratioBox .= '"><span class="intrinsic-ratio-helper" style="padding-bottom: ';
                        $replaceHTML = $ratioBox . (($height[1] / $width[1]) * 100) . '%;"></span>'
                            . $replaceHTML . '</span>';
                    }
                }

                array_push( $search, $imgHTML );
                array_push( $replace, $replaceHTML );
            }
        }

        $content = str_replace( $search, $replace, $content );

        return $content;
    }

    public function filter_avatar( $content ) {
        return $this->filter_images( $content, 'noratio' );
    }

    public function filter_iframes( $html ) {
        return false === strpos( $html, 'iframe' ) ? $html : $this->_add_class( $html, 'lazyload' );
    }

    public function admin_scripts($hook) {
    	
    	$plugin_dir = dirname( __FILE__ );

        if ( 'options-media.php' === $hook || 'upload.php' === $hook ) {

        	wp_enqueue_style('wpimage_admin_style', plugins_url( 'css/admin.css', $plugin_dir ) );
            wp_enqueue_style('tipsy-style', plugins_url( 'css/tipsy.css', $plugin_dir ) );
            wp_enqueue_style('modal-style', plugins_url( 'css/jquery.modal.css', $plugin_dir ) );

            wp_enqueue_script('jquery');
            wp_enqueue_script('tipsy-js', plugins_url( 'js/jquery.tipsy.js', $plugin_dir ), array( 'jquery' ) );
            wp_enqueue_script('async-js', plugins_url( 'js/async.js', $plugin_dir ) );
            wp_enqueue_script('modal-js', plugins_url( 'js/jquery.modal.min.js', $plugin_dir ), array( 'jquery' ) );
            wp_enqueue_script('ajax-script', plugins_url( 'js/ajax.js', $plugin_dir ), array( 'jquery' ) );

            wp_localize_script('ajax-script', 'ajax_object', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );
         }

         if ( 'toplevel_page_optimisationio-dashboard' === $hook ) {
             wp_enqueue_script('stripe-js', 'https://checkout.stripe.com/checkout.js');
             wp_register_script('stripe-custom-js', plugins_url( 'js/stripe.js', $plugin_dir ), 20, "1.3", true );
             wp_enqueue_script('stripe-custom-js');
         }
    }

    public function get_api_status() {
        return true;
    }

    /**
     *  Handles optimizing already-uploaded images in the  Media Library
     */
    public function wpimage_media_library_ajax_callback() {

        $post_req = $_POST;     // Input var okay.

        $image_id = (int) $post_req['id'];
        $type     = isset( $post_req['type'] ) ? $post_req['type'] : false;
        $already_optimised = isset( $post_req['optimised'] ) ? (int) $post_req['optimised'] : 0;
        $this->id = $image_id;

        if ( wp_attachment_is_image( $image_id ) ) {

            //error_reporting(E_ALL);
            //ini_set("display_errors",1);

            $image_path = false;
            $original_image_path = get_attached_file( $image_id );

            if( $already_optimised ){
                $wp_meta = wp_get_attachment_metadata( $image_id );
                if( isset( $wp_meta['file'] ) ){
                    $uploads = wp_upload_dir();
                    $image_path = $uploads['basedir'] . '/optimisationio_media_backup/' . basename( $wp_meta['file'] );
                }
            }

            $image_path = $image_path ? $image_path : $original_image_path;

            $status = $this->get_api_status();

            if ( false === $status ) {
                $kv['error'] = 'There is a problem with your credentials. Please check them in the wp-image.co.uk settings section of Media Settings, and try again.';
                update_post_meta( $image_id, '_wpimage_size', $kv );
                echo json_encode(array( 'error' => $kv['error'] ) );
                exit;
            }

            $result = $this->optimize_image( $image_path, $already_optimised ? false : $this->backup_before_compression );

            if ( $result && $result['url'] ) {

            	$savings_percentage = (int) $result['saved_bytes'] / (int) $result['original_size'] * 100;

                $kv = array(
                    'success' => true,
                    'meta' => wp_get_attachment_metadata( $image_id ),
                    'original_size' => self::pretty_kb( $result['original_size'] ),
                    'compressed_size' => self::pretty_kb( $result['bytes'] ),
                    'saved_bytes' => self::pretty_kb( $result['saved_bytes'] ),
                    'savings_percent' => round( $savings_percentage, 2 ) . '%',
                    'backup_before_compression' => $result['backup_before_compression']
                );

                if ( $this->replace_image( $original_image_path, $result['url'] ) ) {

                    // Get metadata for thumbnails.
                    /*$image_data = wp_get_attachment_metadata( $image_id );
                    $this->optimize_thumbnails( $image_data );*/
                    
                    // Store compressed info to DB.
                    update_post_meta( $image_id, '_wpimage_size', $kv );

                    // Compress thumbnails, store that data too.
                    /*$compressed_thumbs_data = get_post_meta($image_id, '_compressed_thumbs', true);
                    if (!empty($compressed_thumbs_data)) {
                        $kv['thumbs_data'] = $compressed_thumbs_data;
                    }*/
                }
                else {
                    $kv = array( 'error' => 'Could not overwrite original file. Please ensure that your files are writable by plugins.' );
                }

                $this->delete_resources( $result['public_id'] );
                print_r( json_encode( $kv ) );
            }
            else {

                // error or no optimization
                if ( file_exists( $image_path ) ) {

                    $kv = array(
                        'original_size' => self::pretty_kb( filesize( $image_path ) ),
                        'error' => $result['error'],
                        // 'type' => $result['type'],
                    );

                    if( 'This image can not be optimized any further' === $kv['error'] ){
                        $kv['compressed_size'] = 'No savings found';
                        $kv['no_savings'] = true;
                    }

                    update_post_meta( $image_id, '_wpimage_size', $kv );
                }
                else {
                    // file not found
                }

                print_r( json_encode( $result ) );
            }
        }
        
        die();
    }

    /**
     *  Handles optimizing images uploaded through any of the media uploaders.
     */
    public function wpimage_media_uploader_callback($image_id) {

        $this->id = $image_id;

        if (wp_attachment_is_image($image_id)) {

            $settings                  = $this->compression_settings;
            $backup_before_compression = $settings['backup_before_compression'];
            $image_path                = get_attached_file($image_id);
            $result                    = $this->optimize_image($image_path, $backup_before_compression);

            if ($result && isset($result['url'])) {

                $compressed_url                  = $result['url'];
                $savings_percentage              = (int) $result['saved_bytes'] / (int) $result['original_size'] * 100;
                $kv['original_size']             = self::pretty_kb($result['original_size']);
                $kv['compressed_size']           = self::pretty_kb($result['bytes']);
                $kv['saved_bytes']               = self::pretty_kb($result['saved_bytes']);
                $kv['savings_percent']           = round($savings_percentage, 2) . '%';
                $kv['backup_before_compression'] = $settings['backup_before_compression'];
                $kv['success']                   = true;
                $kv['meta']                      = wp_get_attachment_metadata($image_id);

                if ($this->replace_image($image_path, $compressed_url)) {
                    update_post_meta($image_id, '_wpimage_size', $kv);
                    $this->delete_resources($result['public_id']);
                } else {
                    // writing image failed
                    $this->delete_resources($result['public_id']);
                }
            } else {

                // error or no optimization
                if (file_exists($image_path)) {

                    $kv['original_size'] = self::pretty_kb(filesize($image_path));
                    $kv['error']         = $result['error'];
                    $kv['type']          = $result['type'];

                    if ($kv['error'] == 'This image can not be optimized any further') {
                        $kv['compressed_size'] = 'No savings found';
                        $kv['no_savings']      = true;
                    }

                    update_post_meta($image_id, '_wpimage_size', $kv);
                } else {
                    // file not found
                }
            }
        }
    }

    public function show_credentials_validity() {

        $settings   = $this->compression_settings;
        $status = $this->get_api_status();
        $url    = admin_url() . 'images/';

        if ($status !== false && isset($status['active'])) {
            $url .= 'yes.png';
            echo '<p class = "apiStatus">Your credentials are valid <span class = "apiValid" style = "background:url(' . "'$url') no-repeat 0 0" . '"></span></p>';
        } else {
            $url .= 'no.png';
            echo '<p class = "apiStatus">There is a problem with your credentials <span class = "apiInvalid" style = "background:url(' . "'$url') no-repeat 0 0" . '"></span></p>';
        }
    }

    public function validate_options($input) {

        $valid              = array();
        $error              = '';
        $valid['api_lossy'] = $input['api_lossy'];

        if (!function_exists('curl_exec')) {
            $error = 'cURL not available. Wp image compression requires cURL in order to communicate with wp-image.co.uk servers. <br /> Please ask your system administrator or host to install PHP cURL, or contact support@wp-image.co.uk for advice';
        }
        else {
            $status = $this->get_api_status();

            if ($status !== false) {

                if (isset($status['active'])) {
                    if ($status['plan_name'] === 'Developers') {
                        $error = 'Developer API credentials cannot be used with this plugin.';
                    } else {
                        $valid['api_key']    = $input['api_key'];
                        $valid['api_secret'] = $input['api_secret'];
                    }
                } else {
                    $error = 'There is a problem with your credentials. Please check them from your wp-image.co.uk account.';
                }
            } else {
                $error = 'Please enter a valid wp-image.co.uk API key and secret';
            }
        }

        if (!empty($error)) {
            add_settings_error(
                'media', 'api_key_error', $error, 'error'
            );
        }
        return $valid;
    }

    public function show_lossy() {

        $options = get_option('_wpimage_options');
        $value   = isset($options['api_lossy']) ? $options['api_lossy'] : 'lossy';

        $html = '<input type="radio" id="wpicompressor_lossy" name="_wpimage_options[api_lossy]" value="lossy"' . checked('lossy', $value, false) . '/>';
        $html .= '<label for="wpicompressor_lossy">Lossy</label>';

        $html .= '<input style="margin-left:10px;" type="radio" id="wpimage_lossless" name="_wpimage_options[api_lossy]" value="lossless"' . checked('lossless', $value, false) . '/>';
        $html .= '<label for="wpimage_lossless">Lossless</label>';

        echo $html;
    }

    public function add_media_columns($columns) {
        $columns['original_size']   = 'Original Size';
        $columns['compressed_size'] = 'Compressed Size';
        return $columns;
    }

    public function fill_media_columns($column_name, $id) {

        $original_size = filesize( get_attached_file( $id ) );
        $original_size = self::pretty_kb( $original_size );

        $options = get_option('_wpimage_options');
        $type    = isset( $options['api_lossy'] ) ? $options['api_lossy'] : 'lossy';

        if ( 0 === strcmp( $column_name, 'original_size' ) ) {
            if ( wp_attachment_is_image( $id ) ) {
                $meta = get_post_meta( $id, '_wpimage_size', true );
                echo isset( $meta['original_size'] ) ? $meta['original_size'] : $original_size;
            }
            else {
                echo $original_size;
            }
        }
        else if ( 0 === strcmp( $column_name, 'compressed_size' ) ) {

            if ( wp_attachment_is_image( $id ) ) {

                $meta = get_post_meta( $id, '_wpimage_size', true );

                $backup_file_exists = false;
                if( isset( $meta['meta'] ) ) {

                    $uploads = wp_upload_dir();
                    $image_in_backup = $uploads['basedir'] . '/optimisationio_media_backup/';

                    if( isset( $meta['meta']['file'] ) ){
                        $image_in_backup .= basename( $meta['meta']['file'] );
                        $backup_file_exists = file_exists( $image_in_backup );
                    }
                    else{

                        $wp_meta = wp_get_attachment_metadata( $id );

                        if( isset( $wp_meta['file'] ) ){
                            $image_in_backup .= basename( $wp_meta['file'] );
                            $backup_file_exists = file_exists( $image_in_backup );
                        }
                    }
                }

                $image_url = wp_get_attachment_url( $id );
                $filename  = basename( $image_url );

                if ( isset( $meta['compressed_size'] ) ) {

                    $compressed_size = $meta['compressed_size'];

                    if ( $meta['savings_percent'] >= 0 ) {
                        $savings_percentage = $meta['savings_percent'];
                        echo '<strong>' . $compressed_size . '</strong><br/><small>Savings:&nbsp;' . $savings_percentage . '</small>';

                        $thumbs_data  = get_post_meta($id, '_compressed_thumbs', true);
                        $thumbs_count = count( $thumbs_data );

                        if ( ! empty( $thumbs_data ) ) {
                            echo '<small>' . $thumbs_count . ' thumbs optimized' . '</small>';
                        }
                    }
                    else {
                        echo '<br/><small>No further optimization required</small>';
                    }

                    if ( ! empty( $meta['no_savings'] ) ) {
                        echo '<div class="noSavings"><strong>No savings found</strong><br /></div>';
                    }
                    else if ( isset( $meta['error'] ) ) {
                        $error = $meta['error'];
                        echo '<div class="wpimageErrorWrap"><a class="wpimageError" title="' . $error . '">Failed! Hover here</a></div>';
                    }
                }

                if ( ! isset( $meta['compressed_size'] ) || $backup_file_exists ) {
                    echo '<div class="buttonWrap">';
                    echo $backup_file_exists ? '<br/>' : '';
                    echo '<button data-setting="' . $type . '" type="button" class="wpimage_req" data-id="' . $id . '" id="wpimageid-' . $id . '" data-filename="' . $filename . '" data-url="' . $image_url . '" data-optimised="' . ( $backup_file_exists ? 1 : 0 ) . '">';
                    echo $backup_file_exists ? 'Re-Optimize This Image' : 'Optimize This Image';
                    echo '</button>';
                    echo '<span class="wpimageSpinner"></span>';
                    echo '</div>';
                }

                // Is it optimized? Show some stats
                /*if ( isset($meta['compressed_size']) && empty($meta['no_savings'])) {
                    $compressed_size = $meta['compressed_size'];
                    if ($meta['savings_percent'] >= 0) {
                        $savings_percentage = $meta['savings_percent'];
                        echo '<strong>' . $compressed_size . '</strong><br /><br /><small>Savings:&nbsp;' . $savings_percentage . '</small>';

                        $thumbs_data  = get_post_meta($id, '_compressed_thumbs', true);
                        $thumbs_count = count($thumbs_data);

                        if (!empty($thumbs_data)) {
                            echo '<br /><small>' . $thumbs_count . ' thumbs optimized' . '</small>';
                        }
                    } else {
                        echo '<small>No further optimization required</small>';
                    }
                    // Were there no savings, or was there an error?
                } else {
                    $image_url = wp_get_attachment_url($id);
                    $filename  = basename($image_url);
                    echo '<div class="buttonWrap"><button data-setting="' . $type . '" type="button" class="wpimage_req" data-id="' . $id . '" id="wpimageid-' . $id . '" data-filename="' . $filename . '" data-url="' . $image_url . '">Optimize This Image</button><span class="wpimageSpinner"></span></div>';
                    if (!empty($meta['no_savings'])) {
                        echo '<div class="noSavings"><strong>No savings found</strong><br /></div>';
                    } else if (isset($meta['error'])) {
                        $error = $meta['error'];
                        echo '<div class="wpimageErrorWrap"><a class="wpimageError" title="' . $error . '">Failed! Hover here</a></div>';
                    }
                }*/

            }
            else {
                echo 'n/a';
            }
        }
    }

    public function replace_image( $image_path, $compressed_url ) {
    	return wpimages_replace_media_image( $image_path, $compressed_url );
    }

    public function optimize_image($image_path, $backup) {
        return wpimages_optimize_image($image_path, $backup);
    }

    public function delete_resources($public_id) {
        $settings = $this->compression_settings;
        $wpimage  = new Wpimage();
        $wpimage->deleteResource( $public_id );
    }

    public function optimize_thumbnails($image_data) {

        $image_id = $this->id;

        if (empty($image_id)) {
            global $wpdb;
            $post     = $wpdb->get_row($wpdb->prepare("SELECT post_id FROM $wpdb->postmeta WHERE meta_value = %s LIMIT 1", $image_data['file']));
            $image_id = $post->post_id;
        }

        $path_parts = pathinfo($image_data['file']);

        // e.g. 04/02, for use in getting correct path or URL
        $upload_subdir = $path_parts['dirname'];

        $upload_dir = wp_upload_dir();

        // all the way up to /uploads
        $upload_base_path = $upload_dir['basedir'];
        $upload_full_path = $upload_base_path . '/' . $upload_subdir;

        $sizes = array();

        if (isset($image_data['sizes'])) {
            $sizes = $image_data['sizes'];
        }

        if (!empty($sizes)) {

            $thumb_path = '';

            $thumbs_optimized_store = array();
            $this_thumb             = array();

            foreach ($sizes as $key => $size) {

                $thumb_path = $upload_full_path . '/' . $size['file'];

                if (file_exists($thumb_path) !== false) {

                    $result = $this->optimize_image($thumb_path, $this->backup_before_compression);

                    if (!empty($result) && isset($result) && isset($result['url'])) {
                        $compressed_url = $result["url"];
                        if ($this->replace_image($thumb_path, $compressed_url)) {
                            $this_thumb               = array('thumb' => $key, 'file' => $size['file'], 'original_size' => $result['original_size'], 'compressed_size' => $result['bytes'], 'backup_before_compression' => $this->backup_before_compression);
                            $thumbs_optimized_store[] = $this_thumb;
                        }
                        //deleting the image from clould
                        $this->delete_resources($result['public_id']);
                    }
                }
            }
        }
        if (!empty($thumbs_optimized_store)) {
            update_post_meta($image_id, '_compressed_thumbs', $thumbs_optimized_store, false);
        }
        return $image_data;
    }

    public function exe_before_wpheader(){

        if( is_admin() && current_user_can('manage_options') ){

            $server = $_SERVER;   // Input var okay.
            $post_req = 'post' === sanitize_key( $server['REQUEST_METHOD'] )  ? $_POST : array();   // Input var okay.

            if( isset( $post_req['optimisation_save_image_compression_settings'] ) ){   // NOTE:Save ONLY on form submit. Deny access from another call (eg. AJAX).

                if( isset( $post_req['optimisationio_image_compression_settings'] ) && wp_verify_nonce( $post_req['optimisationio_image_compression_settings'], 'optimisationio-image-compression-settings' ) ) {

                    // Image Compression settings.
                    update_option( 'wpimages_max_height', isset( $post_req['wpimages_max_height'] ) ? $post_req['wpimages_max_height'] : WPIMAGE_DEFAULT_MAX_HEIGHT );
                    update_option( 'wpimages_max_width', isset( $post_req['wpimages_max_width'] ) ? $post_req['wpimages_max_width'] : WPIMAGE_DEFAULT_MAX_WIDTH );
                    update_option( 'wpimages_max_height_library', isset( $post_req['wpimages_max_height_library'] ) ? $post_req['wpimages_max_height_library'] : WPIMAGE_DEFAULT_MAX_WIDTH );
                    update_option( 'wpimages_max_width_library', isset( $post_req['wpimages_max_width_library'] ) ? $post_req['wpimages_max_width_library'] : WPIMAGE_DEFAULT_MAX_WIDTH );
                    update_option( 'wpimages_max_height_other', isset( $post_req['wpimages_max_height_other'] ) ? $post_req['wpimages_max_height_other'] : WPIMAGE_DEFAULT_MAX_WIDTH );
                    update_option( 'wpimages_max_width_other', isset( $post_req['wpimages_max_width_other'] ) ? $post_req['wpimages_max_width_other'] : WPIMAGE_DEFAULT_MAX_WIDTH );
                    update_option( 'wpimages_bmp_to_jpg', isset( $post_req['wpimages_bmp_to_jpg'] ) && 1 === (int) $post_req['wpimages_bmp_to_jpg'] ? 1 : 0 );
                    update_option( 'wpimages_png_to_jpg', isset( $post_req['wpimages_png_to_jpg'] ) && 1 === (int) $post_req['wpimages_png_to_jpg'] ? 1 : 0 );
                    update_option( 'wpimages_quality', isset( $post_req['wpimages_quality'] ) ? $post_req['wpimages_quality'] : WPIMAGE_DEFAULT_QUALITY );
                    update_option( 'wpimages_quality_auto', isset( $post_req['wpimages_quality_auto'] ) ? $post_req['wpimages_quality_auto'] : 'auto' );

                    // Lazy Load settings.
                    $lazy_load_values = serialize( $post_req['_wpimage_lazyload'] );
                    update_option('_wpimage_lazyload_options', $lazy_load_values);
                }
            }

            if( isset( $post_req['optimisation_save_cloudinary_api_settings'] ) ){   // NOTE:Save ONLY on form submit. Deny access from another call (eg. AJAX).

                if( isset( $post_req['optimisationio_cloudinary_api_settings'] ) && wp_verify_nonce( $post_req['optimisationio_cloudinary_api_settings'], 'optimisationio-cloudinary-api-settings' ) ) {

                    $enabled = isset( $post_req['custom_cloudinary']['enabled'] ) ? $post_req['custom_cloudinary']['enabled'] : 0;

                    $options = array(
                        'invalid_msg' => '',
                        'success_msg' => '',
                        'cloud_name' => isset( $post_req['custom_cloudinary']['cloud_name'] ) ? sanitize_text_field($post_req['custom_cloudinary']['cloud_name']) : '',
                        'api_key' => isset($post_req['custom_cloudinary']['api_key']) ? sanitize_text_field($post_req['custom_cloudinary']['api_key']) : '',
                        'api_secret' => isset( $post_req['custom_cloudinary']['api_secret'] ) ? sanitize_text_field($post_req['custom_cloudinary']['api_secret']) : '',
                    );

                    if( $enabled ){
                        $valid = $this->validate_cloudinary_credentials( $options['cloud_name'], $options['api_key'], $options['api_secret'] );
                        $options['enabled'] = $enabled && $valid ? 1 : 0;
                        $options['invalid_msg'] = ! $valid ? __('Invalid Cloudinary credentials', 'wpimage') : '';
                        $options['success_msg'] = $valid ? __('Cloudinary credentials validated and saved successful', 'wpimage') : '';
                    }
                    else{
                        $options['enabled'] = 0;
                    }

                    update_option('_wpimage_options_cloudinary', $options );

                    update_option('_wpimage_options', serialize( isset( $post_req['_wpimage_options'] ) ? $post_req['_wpimage_options'] : array() ) );
                }
            }
        }
    }

    public static function pretty_kb($bytes) {
    	return wpimages_pretty_kb($bytes);
    }

    public static function addon_settings(){
    	include dirname( dirname( __FILE__ ) ) . '/views/dashboard-settings.php';
    }

    public static function cloudinary_settings( $include_upgrade = true ){
    	include dirname( dirname( __FILE__ ) ) . '/views/cloudinary-settings.php';
    }
}
